<?xml version="1.0" encoding="UTF-8"?>
<?bluebottle format version="0.1" ?>
<?xml-stylesheet type="text/xsl" href="http://bluebottle.ethz.ch/bluebottle.xsl" ?>
<Text>
<Span style="Normal"><![CDATA[MODULE GUIPatW1R;]]></Span><Span style="Comment"><![CDATA[
	(** 	PURPOSE "Template for the construction of a restorable GUI application which can be started only once.
					The template is a complete, correctly working, correctly terminating application";
		TECHNIQUE "An anchor point winstance is either NIL or points to a window";
		USAGE  "1 - Make a copy of this template and assign it a name of your choice";
				"2 - In the copy, replace all occurences of  "GUIPatW1R" by the name of the new module.
					 Use the [Search] button in the tool bar to access the Search/Replace facility";
				"3 - Remove all comments and add comments about the new application to develop";
				"4 - Blue text denotes, by an unwritten convention, debugging statements (see Text Styles),
					but here additions required for restoring an application appear in blue.";
		AUTHOR "afi";
	 *)]]></Span><Span style="Normal"><![CDATA[

IMPORT
	AosModules, WMComponents, WM := WMWindowManager, WMMessages, WMStandardComponents,
	Utilities]]></Span><Span style="Debug"><![CDATA[, WMRestorable]]></Span><Span style="Normal"><![CDATA[
	;

TYPE
	]]></Span><Span style="Bold"><![CDATA[Window* = OBJECT (WMComponents.FormWindow)]]></Span><Span style="Normal"><![CDATA[

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE CreateForm () : WMComponents.VisualComponent;]]></Span><Span style="Normal"><![CDATA[
		VAR panel : WMStandardComponents.Panel;
		BEGIN
			NEW (panel); panel.bounds.SetExtents (600, 400); panel.fillColor.Set (0FFFFFFFFH);
			
			RETURN panel
		END CreateForm;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE &New (]]></Span><Span style="Debug"><![CDATA[c : WMRestorable.Context]]></Span><Span style="Bold"><![CDATA[);
]]></Span><Span style="Normal"><![CDATA[		VAR vc : WMComponents.VisualComponent;]]></Span><Span style="Comment"><![CDATA[
]]></Span><Span style="Normal"><![CDATA[		BEGIN
			vc := CreateForm ();
			Init (vc.bounds.GetWidth (), vc.bounds.GetHeight (), FALSE);
			SetContent (vc);
]]></Span><Span style="Debug"><![CDATA[			IF c # NIL THEN 
				(* restore the desktop *)
				WMRestorable.AddByContext (SELF, c, {WM.FlagFrame});
				IF c.appData # NIL THEN
					(* Insert application code *)
				END			
			ELSE 
]]></Span><Span style="Normal"><![CDATA[				WM.DefaultAddWindow (SELF)			
]]></Span><Span style="Debug"><![CDATA[			END;
]]></Span><Span style="Normal"><![CDATA[			SetTitle (Utilities.NewString ("GUIPatW1R"));		
		END New;

		]]></Span><Span style="Bold"><![CDATA[PROCEDURE Close ();]]></Span><Span style="Normal"><![CDATA[
		BEGIN

			Close^;
			winstance := NIL
		END Close;
		
]]></Span><Span style="Debug"><![CDATA[		PROCEDURE Handle(VAR x: WMMessages.Message);
		BEGIN
			IF (x.msgType = WMMessages.MsgExt) & (x.ext # NIL) THEN
				IF (x.ext IS WMRestorable.Storage) THEN
					x.ext(WMRestorable.Storage).Add("GUIPatW1R", "GUIPatW1R.Restore", SELF, NIL)
				ELSE Handle^(x)
				END
			ELSE Handle^(x)
			END
		END Handle;
]]></Span><Span style="Normal"><![CDATA[
	END Window;

VAR winstance : Window;

]]></Span><Span style="Comment"><![CDATA[(** Standard single window, restorable. *)]]></Span><Span style="Bold"><![CDATA[
PROCEDURE Open* (par : ANY) : ANY;]]></Span><Span style="Normal"><![CDATA[
BEGIN
	IF winstance = NIL THEN NEW (winstance]]></Span><Span style="Debug"><![CDATA[, NIL]]></Span><Span style="Normal"><![CDATA[) END;
	RETURN NIL
END Open;
]]></Span><Span style="Bold"><![CDATA[
]]></Span><Span style="Debug"><![CDATA[PROCEDURE Restore* (par : ANY) : ANY;
BEGIN
	IF (par # NIL) & (par IS WMRestorable.Context) THEN
		NEW (winstance, par (WMRestorable.Context))
	END;
	RETURN NIL
END Restore;
]]></Span><Span style="Bold"><![CDATA[
PROCEDURE Cleanup;
]]></Span><Span style="Normal"><![CDATA[BEGIN]]></Span><Span style="Bold"><![CDATA[
]]></Span><Span style="Normal"><![CDATA[	IF winstance # NIL THEN winstance.Close END
END Cleanup;

BEGIN
	AosModules.InstallTermHandler (Cleanup);

	]]></Span><Span style="Comment"><![CDATA[(* Additional BEGIN processing. *)]]></Span><Span style="Normal"><![CDATA[
END GUIPatW1R.

GUIPatW1R.Open ~
S.Free GUIPatW1R ~

Remark:

A very small number of programs in the delivery appeared without an exported Release procedure.
These programs willprogressively be replaced by corrected versions with an explicit Restore.
The same functionality was embedded in the Open procedure in the following manner:
]]></Span><Span style="Bold"><![CDATA[
PROCEDURE Open* (par : ANY) : ANY;]]></Span><Span style="Normal"><![CDATA[
BEGIN
	IF winstance # NIL THEN RETURN NIL END;
]]></Span><Span style="Debug"><![CDATA[	IF (par # NIL) & (par IS WMRestorable.Context) THEN
		NEW (winstance, par (WMRestorable.Context)) 
	ELSE ]]></Span><Span style="Normal"><![CDATA[NEW (winstance]]></Span><Span style="Debug"><![CDATA[, NIL]]></Span><Span style="Normal"><![CDATA[)		
]]></Span><Span style="Debug"><![CDATA[	END;
]]></Span><Span style="Normal"><![CDATA[	RETURN NIL
END Open;

and the Handle procedure invoked Open:
]]></Span><Span style="Debug"><![CDATA[					x.ext(WMRestorable.Storage).Add("GUIPatW1R", "GUIPatW1R.Open", SELF, NIL)
]]></Span><Span style="Normal"><![CDATA[
The double meaning of Open is inappropriate. It is preferable to use a Restore procedure
clearly indicating that the windows are restorable.
	
]]></Span>

</Text>
